"use strict";
/*
 * adonis-fcm
 *
 * (c) Lookin Anton <lookin@lookinlab.ru>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_gcm_1 = require("node-gcm");
const Fake_1 = __importDefault(require("./Fake"));
/**
 * FCM class is used to grab an instance of sender
 *
 * @class FCM
 * @constructor
 */
class FCM {
    constructor($config) {
        this.$config = $config;
        this.$fake = null;
        this.$sender = new node_gcm_1.Sender(this.$config.apiKey, this.$config.requestOptions);
        return new Proxy(this, proxyHandler);
    }
    getFake() {
        return this.$fake;
    }
    message(options) {
        return (options instanceof node_gcm_1.Message) ? options : new node_gcm_1.Message(options);
    }
    fake() {
        this.$fake = new Fake_1.default();
    }
    restore() {
        this.$fake = null;
    }
    send(message, recipient, options) {
        return new Promise((resolve, reject) => {
            let callback = function (err, response) {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(response);
                }
            };
            if (!options || typeof options === 'function') {
                options = callback;
                callback = null;
            }
            this.$sender.send(this.message(message), recipient, options, callback);
        });
    }
    sendNoRetry(message, recipient) {
        return new Promise((resolve, reject) => {
            this.$sender.sendNoRetry(this.message(message), recipient, function (err, response) {
                if (err) {
                    reject(err);
                }
                else {
                    resolve(response);
                }
            });
        });
    }
}
exports.default = FCM;
const proxyHandler = {
    get(target, name) {
        if (typeof name === 'symbol' || name === 'inspect') {
            return target[name];
        }
        if (target.$fake && name in target.$fake) {
            return typeof target.$fake[name] === 'function'
                ? target.$fake[name].bind(target.$fake)
                : target.$fake[name];
        }
        return target[name];
    },
};
